import { GraphQLSchema, Kind } from 'graphql';
import type { AST } from 'eslint';
import { Source as LoaderSource } from '@graphql-tools/utils';
import { GraphQLESLintRuleContext } from './types';
import { SiblingOperations } from './sibling-operations';
import type * as ESTree from 'estree';
export declare function requireSiblingsOperations(ruleName: string, context: GraphQLESLintRuleContext): SiblingOperations | never;
export declare function requireGraphQLSchemaFromContext(ruleName: string, context: GraphQLESLintRuleContext): GraphQLSchema | never;
export declare const logger: {
    error: (...args: any[]) => void;
    warn: (...args: any[]) => void;
};
export declare const normalizePath: (path: string) => string;
/**
 * https://github.com/prettier/eslint-plugin-prettier/blob/76bd45ece6d56eb52f75db6b4a1efdd2efb56392/eslint-plugin-prettier.js#L71
 * Given a filepath, get the nearest path that is a regular file.
 * The filepath provided by eslint may be a virtual filepath rather than a file
 * on disk. This attempts to transform a virtual path into an on-disk path
 */
export declare const getOnDiskFilepath: (filepath: string) => string;
export declare const getTypeName: (node: any) => any;
export declare const loaderCache: Record<string, LoaderSource[]>;
export declare const TYPES_KINDS: readonly [Kind.OBJECT_TYPE_DEFINITION, Kind.INTERFACE_TYPE_DEFINITION, Kind.ENUM_TYPE_DEFINITION, Kind.SCALAR_TYPE_DEFINITION, Kind.INPUT_OBJECT_TYPE_DEFINITION, Kind.UNION_TYPE_DEFINITION];
export declare type CaseStyle = 'camelCase' | 'PascalCase' | 'snake_case' | 'UPPER_CASE' | 'kebab-case';
export declare const camelCase: (str: string) => string;
export declare const convertCase: (style: CaseStyle, str: string) => string;
export declare function getLocation(start: ESTree.Position, fieldName?: string): AST.SourceLocation;
export declare const REPORT_ON_FIRST_CHARACTER: {
    column: number;
    line: number;
};
export declare const ARRAY_DEFAULT_OPTIONS: {
    readonly type: "array";
    readonly uniqueItems: true;
    readonly minItems: 1;
    readonly items: {
        readonly type: "string";
    };
};
