"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _path = _interopRequireDefault(require("path"));

var _getExportedName = require("../utilities/getExportedName");

var _isIgnoredFilename = require("../utilities/isIgnoredFilename");

var _parseFilename = require("../utilities/parseFilename");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * @file Rule to ensure that filenames match a convention (default: camelCase)
 * @author Stefan Lau
 * @see https://github.com/selaux/eslint-plugin-filenames/blob/32fc70dd7572211d1e5b97e06ec7a005c77fe8d4/lib/rules/match-regex.js
 */
const create = context => {
  // eslint-disable-next-line unicorn/no-unsafe-regex
  const defaultRegexp = /^([\da-z]+)([A-Z][\da-z]+)*$/ug;
  const conventionRegexp = context.options[0] ? new RegExp(context.options[0], 'u') : defaultRegexp;
  const ignoreExporting = context.options[1] ? context.options[1] : false;
  return {
    Program(node) {
      const filename = context.getFilename();

      const absoluteFilename = _path.default.resolve(filename);

      const parsed = (0, _parseFilename.parseFilename)(absoluteFilename);
      const shouldIgnore = (0, _isIgnoredFilename.isIgnoredFilename)(filename);
      const isExporting = Boolean((0, _getExportedName.getExportedName)(node));
      const matchesRegex = conventionRegexp.test(parsed.name);

      if (shouldIgnore) {
        return;
      }

      if (ignoreExporting && isExporting) {
        return;
      }

      if (!matchesRegex) {
        context.report({
          data: {
            name: parsed.base
          },
          message: 'Filename \'{{name}}\' does not match the naming convention.',
          node
        });
      }
    }

  };
};

var _default = {
  create,
  meta: {
    schema: [{
      default: null,
      oneOf: [{
        type: 'string'
      }, {
        type: 'null'
      }]
    }, {
      default: false,
      type: 'boolean'
    }],
    type: 'suggestion'
  }
};
exports.default = _default;
module.exports = exports.default;